module module_wrfinputfile
  use netcdf
  implicit none
  ! Define a data structure for wrfinput file

  type wrfinput_type
    ! Grid (map) definitions
     integer :: grid_id
     integer :: idim
     integer :: jdim
     integer :: map_proj
     real    :: Dx
     real    :: Dy
     real    :: LoV
     real    :: Latin1
     real    :: Latin2
     real    :: La1
     real    :: La2
     real    :: Lo1
     real    :: Lo2
     character(len=4) :: landuse_dataset  ! "MODIS" or "USGS"
     integer :: iswater
     !
     ! Some 2d arrays of grid/geographical information:
     !
     real, pointer, dimension(:,:)   :: lat    ! Grid point latitudes
     real, pointer, dimension(:,:)   :: lon    ! Grid point longitudes
     real, pointer, dimension(:,:)   :: ter    ! Terrain field
     real, pointer, dimension(:,:)   :: use    ! Land-use (vegetation) field
!KWM     real, pointer, dimension(:,:)   :: tmn    ! Deep-soil temperature field
     real, pointer, dimension(:,:,:) :: veg    ! 12-month Green Vegetation Fraction climatology
  end type wrfinput_type

contains

  subroutine error_handler(status, failure, success)
    !
    ! Check the error flag from a NetCDF function call, and print appropriate
    ! error message.
    !
    implicit none
    integer,                    intent(in) :: status
    character(len=*), optional, intent(in) :: failure
    character(len=*), optional, intent(in) :: success

    if (status .ne. NF90_NOERR) then
       if (present(failure)) then
          write(*,'(/," ***** ", A)') failure
       endif
       write(*,'(" ***** ",A,/)') nf90_strerror(status)
       stop 'Stopped'
    endif

    if (present(success)) then
       write(*,'(A)') success
    endif

  end subroutine error_handler

  
end module module_wrfinputfile
