program fluxes_statistics
  use kwm_date_utilities
  use kwm_grid_utilities
  use kwm_plot_utilities
  use module_hd
!  use v3_module
  implicit none

  character(len=256) :: flnm
  character(len=16) :: startdate = "2002-05-14_00:00"
  character(len=16) :: enddate =   "2002-06-24_00:00"

  integer :: ntim
  integer :: ista

  integer, parameter :: MAXSTA = 10
  ! integer, parameter :: MAXTIM = 9500
  integer, parameter :: MAXTIM = 4500
  real, dimension(MAXSTA) :: lat, lon, x, y
  real, dimension(MAXSTA, MAXTIM) :: hfx = -1.E33
  real, dimension(MAXSTA, MAXTIM) :: qfx_old = -1.E33
  real, dimension(MAXSTA, MAXTIM) :: qfx_corrected = -1.E33
  real, dimension(MAXSTA, MAXTIM) :: hrldas_hfx = -1.E33
  real, dimension(MAXSTA, MAXTIM) :: hrldas_qfx = -1.E33
  real, pointer, dimension(:,:) :: hfxptr, qfxptr, vegtyp, soltyp

  integer :: ierr, ibf, iaf, ihr
  character(len=16) :: nowdate
  integer, parameter :: icode = 10, iunit = 10

  character(len=16), dimension(MAXSTA,MAXTIM) :: hdate, hrldas_hdate
  integer :: itime
  logical :: lexist, pendown

  integer, dimension(0:23) :: qfxcount = 0, station_qfxcount, qfxcount_old = 0, station_qfxcount_old
  integer, dimension(0:23) :: hfxcount = 0, station_hfxcount

  real, dimension(0:23) :: qfxrmse = 0, qfxbias = 0, qfxoavg = 0, qfxmavg = 0 
  real, dimension(0:23) :: qfxrmse_old = 0, qfxbias_old = 0, qfxoavg_old = 0, qfxmavg_old = 0 
  real, dimension(0:23) :: hfxrmse = 0, hfxbias = 0, hfxoavg = 0, hfxmavg = 0

  real, dimension(0:23) :: station_qfxrmse, station_qfxbias, station_qfxoavg, station_qfxmavg
  real, dimension(0:23) :: station_qfxrmse_old, station_qfxbias_old, station_qfxoavg_old, station_qfxmavg_old

  real, dimension(0:23) :: station_hfxrmse, station_hfxbias, station_hfxoavg, station_hfxmavg

  real :: station_qfx_bias, station_qfxcorr_bias, station_hfx_bias
  real :: station_qfx_rmse, station_qfxcorr_rmse, station_hfx_rmse
  integer :: station_hfx_count, station_qfx_count, station_qfxcorr_count

  type(nf_structure) :: nfstruct
  real :: xl, xr, xb, xt, wl, wr, wb, wt
  integer :: ml

  character(len=256) :: fluxobs_directory = ""
  character(len=256) :: ldasout_directory = ""

  call read_namelist(fluxobs_directory, ldasout_directory)

!KWM  open(iunit, file="mem.dump", form='unformatted')
!KWM  close(iunit, status="delete")
!KWM  inquire (file="mem.dump", exist=lexist)
!KWM
!KWM  if (.not. lexist) then

     call geth_idts(enddate, startdate, ntim)
     ntim = (ntim / 15) + 1
     print*, 'ntim = ', ntim

     ! Read the IHOP site data into arrays

     do ista = 1, maxsta
        call read_ihop(fluxobs_directory, startdate, ista, lat(ista), lon(ista),  &
             hdate(ista,:), hfx(ista,:), qfx_old(ista,:), qfx_corrected(ista,:), maxtim)
     enddo

     ! Read the HRLDAS (NetCDF-format) data into arrays
     call geth_newdate(nowdate(1:16), startdate(1:16), -60)

     do while ( nowdate <= enddate )
        call geth_newdate(nowdate, nowdate, 60)
        print*, 'nowdate = ', nowdate
!        flnm = "/avn2/kmanning/T1_TEST/Run/"&
!        flnm = "/avn2/kmanning/STATS/LDASOUT/"&
        flnm = trim(ldasout_directory)//"/" &
             //nowdate(1:4)//nowdate(6:7)// &
             nowdate(9:10)//nowdate(12:13)//".LDASOUT_DOMAIN2"
        call netcdf_open(trim(flnm), nfstruct)


!KWM  call kwm_init_ncargks("locations.cgm")
!KWM        call wrfmap(nfstruct)
!KWM        call getset(xl, xr, xb, xt, wl, wr, wb, wt, ml)
!KWM        call set(xl, xr, xb, xt, 1., float(nfstruct%idim), 1., float(nfstruct%jdim), 1)

        if (nowdate == startdate) then
           do ista = 1, maxsta
              call lltoxy_hd(lat(ista), lon(ista), x(ista), y(ista), nfstruct)
!KWM CORRECT OFFSET              ! x and y are in dot-point coordinates.  
!KWM CORRECT OFFSET              ! Convert to cross-point coordinates
!KWM CORRECT OFFSET              x(ista) = x(ista) - 0.5
!KWM CORRECT OFFSET              y(ista) = y(ista) - 0.5
              print*, 'nfstruct%dskm = ', nfstruct%dskm
              print*, 'reflat, reflon = ', nfstruct%reflat, nfstruct%reflon
              print*, 'lat, lon, x, y = ', lat(ista), lon(ista), x(ista), y(ista)
!KWM              call ngdots(x(ista), y(ista), 1, 1.0, color_index("red"))
           enddo
        endif

!KWM  call kwm_close_ncargks

        call netcdf_get_field(nfstruct, "HFX", 1, hfxptr)
        call netcdf_get_field(nfstruct, "QFX", 1, qfxptr)
        call netcdf_get_field(nfstruct, "IVGTYP", 1, vegtyp)
        call netcdf_get_field(nfstruct, "ISLTYP", 1, soltyp)

        ! qfxptr = qfxptr * 2.5E6 ! For some of the older data I have that is in different units

        call geth_idts(nowdate, startdate, itime)
        itime = (itime / 15) + 1
        if (itime > maxtim) stop "Increase maxtim"
        print*, 'itime = ', itime, nowdate

        do ista = 1, maxsta
           ! print*, 'x y = ', x(ista), y(ista)
           hrldas_hdate(ista, itime) = nowdate
!KWM           hrldas_hfx(ista, itime) = hfxptr(nint(x(ista)),nint(y(ista)))
!KWM           hrldas_qfx(ista, itime) = qfxptr(nint(x(ista)),nint(y(ista)))
           hrldas_hfx(ista,itime) = &
                four_point(hfxptr, nfstruct%idim, nfstruct%jdim, x(ista), y(ista))
           hrldas_qfx(ista,itime) = &
                four_point(qfxptr, nfstruct%idim, nfstruct%jdim, x(ista), y(ista))
           ! print*, 'Station, vegetype, soiltype = ', ista,  &
           ! vegtyp(nint(x(ista)), nint(y(ista))), soltyp(nint(x(ista)), nint(y(ista)))
        enddo
        print*, hrldas_hfx(:,itime)

        ierr = nf90_close(nfstruct%ncid)

        deallocate(hfxptr, qfxptr, vegtyp, soltyp)
        nullify(hfxptr, qfxptr, vegtyp, soltyp)

     enddo



     ! Fill in observed flux data on the hour
     do itime = 2, maxtim-1
        do ista = 1, maxsta
           if (qfx_corrected(ista,itime) < -1.E25) then
              if ((qfx_corrected(ista,itime-1) > -1.E25) .and. &
                   (qfx_corrected(ista,itime+1) > -1.E25)) then
                 qfx_corrected(ista,itime) = &
                      0.5*(qfx_corrected(ista,itime-1)+qfx_corrected(ista,itime+1))
              endif
           endif
           if (qfx_old(ista,itime) < -1.E25) then
              if ((qfx_old(ista,itime-1) > -1.E25) .and. &
                   (qfx_old(ista,itime+1) > -1.E25)) then
                 qfx_old(ista,itime) = &
                      0.5*(qfx_old(ista,itime-1)+qfx_old(ista,itime+1))
              endif
           endif
           if (hfx(ista,itime) < -1.E25) then
              if ((hfx(ista,itime-1) > -1.E25) .and. (hfx(ista,itime+1) > -1.E25)) then
                 hfx(ista,itime) = 0.5*(hfx(ista,itime-1)+hfx(ista,itime+1))
              endif
           endif
        enddo
     enddo

     ! Fill in hrldas flux data on the off times
     TIMELOOP : do itime = 2, maxtim-1
        do ista = 1, maxsta
           if (hrldas_hdate(ista,itime) > enddate) exit TIMELOOP
           if (hdate(ista,itime) > enddate) exit TIMELOOP
           print '(A, i6, i6, f12.8, "(",A16,")" 1x, "(",A16, ")")', 'itime, ista = ', &
                itime, ista, hrldas_qfx(ista, itime), hdate(ista, itime), hrldas_hdate(ista, itime)
           if ( hrldas_qfx(ista, itime) < -1.E25 ) then
              ibf = itime
              do while ( hrldas_qfx(ista, ibf) < -1.E25)
                 ibf = ibf - 1
              enddo
              iaf = itime
              do while ( hrldas_qfx(ista, iaf) < -1.E25)
                 iaf = iaf + 1
              enddo
              hrldas_qfx(ista,itime) = (hrldas_qfx(ista,ibf)*(iaf-itime) + &
                   hrldas_qfx(ista,iaf)*(itime-ibf))/float(iaf-ibf)
           endif

           if ( hrldas_hfx(ista, itime) < -1.E25 ) then
              ibf = itime
              do while ( hrldas_hfx(ista, ibf) < -1.E25)
                 ibf = ibf - 1
              enddo
              iaf = itime
              do while ( hrldas_hfx(ista, iaf) < -1.E25)
                 iaf = iaf + 1
              enddo
              hrldas_hfx(ista,itime) = (hrldas_hfx(ista,ibf)*(iaf-itime) + &
                   hrldas_hfx(ista,iaf)*(itime-ibf))/float(iaf-ibf)
           endif

        enddo
     enddo TIMELOOP

!KWM     ! Mem Dump
!KWM     open(114, file='mem.dump', form='unformatted', action='write')
!KWM     write(114) lat, lon, x, y, hrldas_hdate, hrldas_hfx, hrldas_qfx, &
!KWM          hdate, hfx, qfx_old, qfx_corrected, ntim
!KWM     close(114)
!KWM  else
!KWM     open(114, file='mem.dump', form='unformatted', action='read')
!KWM     read(114) lat, lon, x, y, hrldas_hdate, hrldas_hfx, hrldas_qfx, &
!KWM          hdate, hfx, qfx_old, qfx_corrected, ntim
!KWM     close(114)
!KWM  endif

  ! Now that I've got everything, do some plotting:

  call kwm_init_ncargks("fluxes.cgm")

  pendown = .FALSE.
  STATIONLOOP : do ista = 1, maxsta
!     print*, 'ista, qfx_corrected = ', ista, qfx_corrected(ista,:)
     station_qfxoavg = 0.
     station_qfxmavg = 0.
     station_qfxbias = 0.
     station_qfxrmse = 0.
     station_qfxcount = 0

     station_hfxoavg = 0.
     station_hfxmavg = 0.
     station_hfxbias = 0.
     station_hfxrmse = 0.
     station_hfxcount = 0

     station_qfxoavg_old = 0.
     station_qfxmavg_old = 0.
     station_qfxbias_old = 0.
     station_qfxrmse_old = 0.
     station_qfxcount_old = 0

     station_hfx_bias = 0.
     station_qfx_bias = 0.
     station_qfxcorr_bias = 0.

     station_hfx_count = 0.
     station_qfx_count = 0.
     station_qfxcorr_count = 0.

     station_hfx_rmse = 0.
     station_qfx_rmse = 0.
     station_qfxcorr_rmse = 0.

     call gsplci(color_index("black"))
     call set (.15, .9, .1, .9, 0., float(maxtim), -100., 500., 1)
     call periml(0, 0, 0, 0)

     call gsplci(color_index("blue"))
     call gsln(1)
     nowdate = startdate
     
     ! QFX_Corrected
     do while ( nowdate(1:13) < enddate(1:13) )
        call geth_newdate(nowdate(1:13), nowdate(1:13), 1)
        call geth_idts(nowdate(1:13), startdate(1:13), itime)
        itime = (itime * 4) + 1
        print*, 'itime = ', itime
        ! if (qfx_corrected(ista,itime) > -1.E25) then
        if (qfx_corrected(ista,itime) > -2000) then
           read(nowdate(12:13),*) ihr
           qfxoavg(ihr) = qfxoavg(ihr) + qfx_corrected(ista,itime+1)
           qfxmavg(ihr) = qfxmavg(ihr) + hrldas_qfx(ista,itime)
           qfxbias(ihr) = qfxbias(ihr) + (hrldas_qfx(ista,itime)-qfx_corrected(ista,itime+1))
           qfxrmse(ihr) = qfxrmse(ihr) + &
                ((hrldas_qfx(ista,itime) - qfx_corrected(ista,itime+1))**2)
           qfxcount(ihr) = qfxcount(ihr) + 1

           station_qfxoavg(ihr) = station_qfxoavg(ihr) + qfx_corrected(ista,itime+1)
           station_qfxmavg(ihr) = station_qfxmavg(ihr) + hrldas_qfx(ista,itime)
           station_qfxbias(ihr) = station_qfxbias(ihr) + (hrldas_qfx(ista,itime)-qfx_corrected(ista,itime+1))
           station_qfxrmse(ihr) = station_qfxrmse(ihr) + ((hrldas_qfx(ista,itime) - qfx_corrected(ista,itime+1))**2)
           station_qfxcount(ihr) = station_qfxcount(ihr) + 1

           station_qfxcorr_bias = station_qfxcorr_bias + (hrldas_qfx(ista,itime)-qfx_corrected(ista, itime+1))
           station_qfxcorr_rmse = station_qfxcorr_rmse + (hrldas_qfx(ista,itime)-qfx_corrected(ista, itime+1))**2
           station_qfxcorr_count = station_qfxcorr_count + 1

           ! print*, 'qfx_corrected = ', qfx_corrected(ista, itime)
           if (pendown) then
              call vector(float(itime), qfx_corrected(ista,itime))
           else
              call frstpt(float(itime), qfx_corrected(ista,itime))
              pendown = .TRUE.
           endif
        endif
     enddo
     call plotif(0., 0., 2)
     pendown = .FALSE.

     nowdate = startdate
     call gsplci(color_index("green"))
     ! QFX_old
     do while ( nowdate(1:13) < enddate(1:13) )
        call geth_newdate(nowdate(1:13), nowdate(1:13), 1)
        call geth_idts(nowdate(1:13), startdate(1:13), itime)
        itime = (itime * 4) + 1
        if (qfx_old(ista,itime) > -2000) then
           read(nowdate(12:13),*) ihr
           qfxoavg_old(ihr) = qfxoavg_old(ihr) + qfx_old(ista,itime+1)
           qfxmavg_old(ihr) = qfxmavg_old(ihr) + hrldas_qfx(ista,itime)
           qfxbias_old(ihr) = qfxbias_old(ihr) + (hrldas_qfx(ista,itime)-qfx_old(ista,itime+1))
           qfxrmse_old(ihr) = qfxrmse_old(ihr) + &
                ((hrldas_qfx(ista,itime) - qfx_old(ista,itime+1))**2)
           qfxcount_old(ihr) = qfxcount_old(ihr) + 1

           station_qfxoavg_old(ihr) = station_qfxoavg_old(ihr) + qfx_old(ista,itime+1)
           station_qfxmavg_old(ihr) = station_qfxmavg_old(ihr) + hrldas_qfx(ista,itime)
           station_qfxbias_old(ihr) = station_qfxbias_old(ihr) + (hrldas_qfx(ista,itime)-qfx_old(ista,itime+1))
           station_qfxrmse_old(ihr) = station_qfxrmse_old(ihr) + ((hrldas_qfx(ista,itime) - qfx_old(ista,itime+1))**2)
           station_qfxcount_old(ihr) = station_qfxcount_old(ihr) + 1

           station_qfx_bias = station_qfx_bias + (hrldas_qfx(ista,itime) - qfx_old(ista, itime+1))
           station_qfx_rmse = station_qfx_rmse + (hrldas_qfx(ista,itime) - qfx_old(ista, itime+1))**2
           station_qfx_count = station_qfx_count + 1

           ! print*, 'qfx_corrected = ', qfx_corrected(ista, itime)
           if (pendown) then
              call vector(float(itime), qfx_old(ista,itime))
           else
              call frstpt(float(itime), qfx_old(ista,itime))
              pendown = .TRUE.
           endif
        endif
     enddo
     call plotif(0., 0., 2)
     pendown = .FALSE.

     ! HRLDAS qfx
     call gsplci(color_index("red"))
     nowdate = startdate
     do while ( nowdate(1:13) < enddate(1:13) )
        call geth_newdate(nowdate(1:13), nowdate(1:13), 1)
        call geth_idts(nowdate(1:13), startdate(1:13), itime)
        itime = (itime * 4) + 1
        if (hrldas_qfx(ista,itime) > -2000) then
           read(nowdate(12:13),*) ihr

           !print*, hrldas_hdate(ista, itime)(1:13), hdate(ista, itime+1)(1:13)
           if (pendown) then
              call vector(float(itime), hrldas_qfx(ista,itime))
           else
              call frstpt(float(itime), hrldas_qfx(ista,itime))
              pendown = .TRUE.
           endif
        endif
     enddo
     call plotif(0., 0., 2)
     pendown = .FALSE.

     call frame

     station_qfxoavg = station_qfxoavg / float(station_qfxcount)
     station_qfxmavg = station_qfxmavg / float(station_qfxcount)
     station_qfxbias = station_qfxbias / float(station_qfxcount)
     station_qfxrmse = sqrt(station_qfxrmse/float(station_qfxcount))

     station_qfxoavg_old = station_qfxoavg_old / float(station_qfxcount_old)
     station_qfxmavg_old = station_qfxmavg_old / float(station_qfxcount_old)
     station_qfxbias_old = station_qfxbias_old / float(station_qfxcount_old)
     station_qfxrmse_old = sqrt(station_qfxrmse_old/float(station_qfxcount_old))

     call gsplci(color_index("black"))
     call set (.15, .9, .1, .9, 0., 24., -100., 500., 1)
     call periml(0, 0, 0, 0)
  
     call gsplci(color_index("blue"))
     call frstpt(0.0, station_qfxoavg(0))
     do itime = 1, 24
        call vector(float(itime), station_qfxoavg(mod(itime,24)))
     enddo
     call plotif(0.,0.,2)

     call gsplci(color_index("green"))
     call frstpt(0.0, station_qfxoavg_old(0))
     do itime = 1, 24
        call vector(float(itime), station_qfxoavg_old(mod(itime,24)))
     enddo
     call plotif(0.,0.,2)

     call gsplci(color_index("red"))
     call frstpt(0.0, station_qfxmavg(0))
     do itime = 1, 24
        call vector(float(itime), station_qfxmavg(mod(itime,24)))
     enddo
     call plotif(0.,0.,2)

     call frame()

     
     ! HFX
     call gsplci(color_index("black"))
     call set (.15, .9, .1, .9, 0., float(maxtim), -100., 500., 1)
     call periml(0, 0, 0, 0)

     call gsplci(color_index("blue"))

     nowdate = startdate
     pendown = .FALSE.
     do while ( nowdate(1:13) < enddate(1:13) )
        call geth_newdate(nowdate(1:13), nowdate(1:13), 1)
        call geth_idts(nowdate(1:13), startdate(1:13), itime)
        itime = (itime * 4) + 1
        if (hfx(ista,itime) > -2000) then
           read(nowdate(12:13),*) ihr
           hfxoavg(ihr) = hfxoavg(ihr) + hfx(ista,itime+1)
           hfxmavg(ihr) = hfxmavg(ihr) + hrldas_hfx(ista,itime)
           hfxbias(ihr) = hfxbias(ihr) + (hrldas_hfx(ista,itime)-hfx(ista,itime+1))
           hfxrmse(ihr) = hfxrmse(ihr) + &
                ((hrldas_hfx(ista,itime) - hfx(ista,itime+1))**2)
           hfxcount(ihr) = hfxcount(ihr) + 1

           station_hfxoavg(ihr) = station_hfxoavg(ihr) + hfx(ista,itime+1)
           station_hfxmavg(ihr) = station_hfxmavg(ihr) + hrldas_hfx(ista,itime)
           station_hfxbias(ihr) = station_hfxbias(ihr) + (hrldas_hfx(ista,itime)-hfx(ista,itime+1))
           station_hfxrmse(ihr) = station_hfxrmse(ihr) + ((hrldas_hfx(ista,itime) - hfx(ista,itime+1))**2)
           station_hfxcount(ihr) = station_hfxcount(ihr) + 1

           station_hfx_bias = station_hfx_bias + (hrldas_hfx(ista,itime) - hfx(ista, itime+1))
           station_hfx_rmse = station_hfx_rmse + (hrldas_hfx(ista,itime) - hfx(ista, itime+1))**2
           station_hfx_count = station_hfx_count + 1

           if (pendown) then
              call vector(float(itime), hfx(ista,itime))
           else
              call frstpt(float(itime), hfx(ista,itime))
              pendown = .TRUE.
           endif
        endif
     enddo
     call plotif(0., 0., 2)
     pendown = .FALSE.

     ! HRLDAS hfx
     call gsplci(color_index("red"))
     nowdate = startdate
     do while ( nowdate(1:13) < enddate(1:13) )
        call geth_newdate(nowdate(1:13), nowdate(1:13), 1)
        call geth_idts(nowdate(1:13), startdate(1:13), itime)
        itime = (itime * 4) + 1
        if (hrldas_hfx(ista,itime) > -1.E25) then
           read(nowdate(12:13),*) ihr
           !print*, hrldas_hdate(ista, itime)(1:13), hdate(ista, itime+1)(1:13)
           if (pendown) then
              call vector(float(itime), hrldas_hfx(ista,itime))
           else
              call frstpt(float(itime), hrldas_hfx(ista,itime))
              pendown = .TRUE.
           endif
        endif
     enddo
     call plotif(0., 0., 2)
     pendown = .FALSE.

     call frame

     station_hfxoavg = station_hfxoavg / float(station_hfxcount)
     station_hfxmavg = station_hfxmavg / float(station_hfxcount)
     station_hfxbias = station_hfxbias / float(station_hfxcount)
     station_hfxrmse = sqrt(station_hfxrmse/float(station_hfxcount))


     call gsplci(color_index("black"))
     call set (.15, .9, .1, .9, 0., 24., -100., 500., 1)
     call periml(0, 0, 0, 0)
  
     call gsplci(color_index("blue"))
     call frstpt(0.0, station_hfxoavg(0))
     do itime = 1, 24
        call vector(float(itime), station_hfxoavg(mod(itime,24)))
     enddo
     call plotif(0.,0.,2)

     call gsplci(color_index("red"))
     call frstpt(0.0, station_hfxmavg(0))
     do itime = 1, 24
        call vector(float(itime), station_hfxmavg(mod(itime,24)))
     enddo
     call plotif(0.,0.,2)

     call frame()

     write(*,'(I2,2x,8F10.4)') ista, sqrt(station_hfx_rmse/float(station_hfx_count)), &
          sqrt(station_qfx_rmse/float(station_qfx_count)), &
          sqrt(station_qfxcorr_rmse/float(station_qfxcorr_count)), &
          station_hfx_bias/float(station_hfx_count), &
          station_qfx_bias/float(station_qfx_count), &
          station_qfxcorr_bias/float(station_qfxcorr_count)

  enddo STATIONLOOP

  qfxoavg = qfxoavg / float(qfxcount)
  qfxmavg = qfxmavg / float(qfxcount)
  qfxbias = qfxbias / float(qfxcount)
  qfxrmse = sqrt(qfxrmse/float(qfxcount))

  qfxoavg_old = qfxoavg_old / float(qfxcount_old)
  qfxmavg_old = qfxmavg_old / float(qfxcount_old)
  qfxbias_old = qfxbias_old / float(qfxcount_old)
  qfxrmse_old = sqrt(qfxrmse_old/float(qfxcount_old))

  hfxoavg = hfxoavg / float(hfxcount)
  hfxmavg = hfxmavg / float(hfxcount)
  hfxbias = hfxbias / float(hfxcount)
  hfxrmse = sqrt(hfxrmse/float(hfxcount))


  ! Plot our average diurnal cycle.
  
  call gsplci(color_index("black"))
  call set (0., 1., 0., 1., 0., 1., 0., 1., 1)
  call pchiqu(0.5*(.15+.9), 0.93, "Observed QFX (solid), Corrected Observed QFX (dot), HRLDAS QFX (dash)", 0.013, 0., 0.)
  call set (.15, .9, .55, .9, 0., 24., -100., 500., 1)
  call gasetc("XLF", '(I2.2)')
  call gasetc("YLF", '(F5.0)')
  call gaseti("LTY", 2)
  call periml(8, 3, 6, 0)

  call gsplci(color_index("gray80"))
  call line(0., 0., 24., 0.)
  call gsplci(color_index("black"))

  
  call gsln(1)
  !  call gsplci(color_index("green"))
  call frstpt(0.0, qfxoavg_old(0))
  do itime = 1, 24
     call vector(float(itime), qfxoavg_old(mod(itime,24)))
  enddo
  call plotif(0.,0.,2)

  !call gsplci(color_index("blue"))
  call gsln(3)
  call frstpt(0.0, qfxoavg(0))
  do itime = 1, 24
     call vector(float(itime), qfxoavg(mod(itime,24)))
  enddo
  call plotif(0.,0.,2)

  call gsln(2)
!  call gsplci(color_index("red"))
  call frstpt(0.0, qfxmavg(0))
  do itime = 1, 24
     call vector(float(itime), qfxmavg(mod(itime,24)))
  enddo
  call plotif(0.,0.,2)
  call gsln(1)

  call gsplci(color_index("black"))
  call set (0., 1., 0., 1., 0., 1., 0., 1., 1)
  call pchiqu(0.5*(.15+.9), 0.46, "QFX bias w.r.t original obs (solid) and corrected obs (dot)", 0.013, 0., 0.)
  call pchiqu(0.5*(.15+.9), 0.42, "QFX rmse w.r.t original obs (dash) and corrected obs (dot-dash)", 0.013, 0., 0.)
  call set (.15, .9, .05, .40, 0., 24., -100., 500., 1)
  call gasetc("XLF", '(I2.2)')
  call gasetc("YLF", '(F5.0)')
  call gaseti("LTY", 2)
  call periml(8, 3, 6, 0)

  call gsplci(color_index("gray80"))
  call line(0., 0., 24., 0.)
  call gsplci(color_index("black"))

  ! call gsplci(color_index("green"))
  call gsln(1)
  call frstpt(0.0, qfxbias_old(0))
  do itime = 1, 24
     call vector(float(itime), qfxbias_old(mod(itime,24)))
  enddo
  call plotif(0.,0.,2)

  ! call gsplci(color_index("blue"))
  call gsln(3)
  call frstpt(0.0, qfxbias(0))
  do itime = 1, 24
     call vector(float(itime), qfxbias(mod(itime,24)))
  enddo
  call plotif(0.,0.,2)

  call gsln(2)
  call frstpt(0.0, qfxrmse_old(0))
  do itime = 1, 24
     call vector(float(itime), qfxrmse_old(mod(itime,24)))
  enddo
  call plotif(0.,0.,2)

  call gsln(4)
  ! call gsplci(color_index("red"))
  call frstpt(0.0, qfxrmse(0))
  do itime = 1, 24
     call vector(float(itime), qfxrmse(mod(itime,24)))
  enddo
  call plotif(0.,0.,2)
  call gsln(1)

  call frame()

  call gsplci(color_index("black"))
  call set (0., 1., 0., 1., 0., 1., 0., 1., 1)
  call pchiqu(0.5*(.15+.9), 0.93, "Observed HFX (solid) and HRLDAS HFX (dash)", 0.013, 0., 0.)
  call gasetc("XLF", '(I2.2)')
  call gasetc("YLF", '(F5.0)')
  call gaseti("LTY", 2)
  call set (.15, .9, .55, .9, 0., 24., -100., 500., 1)
  call periml(8, 3, 6, 0)

  call gsplci(color_index("gray80"))
  call line(0., 0., 24., 0.)
  call gsplci(color_index("black"))

  
  ! call gsplci(color_index("blue"))
  call gsln(1)
  call frstpt(0.0, hfxoavg(0))
  do itime = 1, 24
     call vector(float(itime), hfxoavg(mod(itime,24)))
  enddo
  call plotif(0.,0.,2)

  call gsln(2)
!  call gsplci(color_index("red"))
  call frstpt(0.0, hfxmavg(0))
  do itime = 1, 24
     call vector(float(itime), hfxmavg(mod(itime,24)))
  enddo
  call plotif(0.,0.,2)
  call gsln(1)

  call gsplci(color_index("black"))
  call set (0., 1., 0., 1., 0., 1., 0., 1., 1)
  call pchiqu(0.5*(.15+.9), 0.43, "HFX bias (solid) and RMSE (dash)", 0.013, 0., 0.)
  call gasetc("XLF", '(I2.2)')
  call gasetc("YLF", '(F5.0)')
  call gaseti("LTY", 2)
  call set (.15, .9, .05, .4, 0., 24., -100., 500., 1)
  call periml(8, 3, 6, 0)

  call gsplci(color_index("gray80"))
  call line(0., 0., 24., 0.)
  call gsplci(color_index("black"))

  
  ! call gsplci(color_index("blue"))
  call gsln(1)
  call frstpt(0.0, hfxbias(0))
  do itime = 1, 24
     call vector(float(itime), hfxbias(mod(itime,24)))
  enddo
  call plotif(0.,0.,2)

  call gsln(2)
  ! call gsplci(color_index("red"))
  call frstpt(0.0, hfxrmse(0))
  do itime = 1, 24
     call vector(float(itime), hfxrmse(mod(itime,24)))
  enddo
  call plotif(0.,0.,2)







  call kwm_close_ncargks()
end program fluxes_statistics

subroutine read_ihop(fluxobs_directory, &
     startdate, ista, lat, lon, hdate, hfx, qfx_old, qfx_corrected, maxtim)
  use kwm_date_utilities
  use kwm_string_utilities
  implicit none
  character(len=*),                     intent(in)  :: fluxobs_directory
  character(len=16),                    intent(in)  :: startdate
  integer,                              intent(in)  :: ista
  integer,                              intent(in)  :: maxtim
  real,                                 intent(out) :: lat
  real,                                 intent(out) :: lon
  character(len=16), dimension(maxtim), intent(out) :: hdate
  real,              dimension(maxtim), intent(out) :: hfx
  real,              dimension(maxtim), intent(out) :: qfx_old
  real,              dimension(maxtim), intent(out) :: qfx_corrected

  character(len=256) :: flnm
  integer, parameter :: iunit = 26
  character(len=16) :: rddate

  integer :: rdyyyy, rdmm, rddd, rdhhmmss
  integer :: ierr, idum, itime
  real :: rd_rlw, rd_rsw

  real :: rt_T, rd_P, rd_MR, rd_SPD, rd_U, rd_V, rd_rainacc, rd_rainr
  real :: rd_Rnet, rd_Rpar, rd_Tsfc, rd_LE, rd_H, rd_Gsfc, rd_LEc
  integer :: indx
  character(len=8) :: Hrd_LEc

  character(len=256) :: string

  if (ista < 10) then
     write(flnm, fmt='(A,"/IHOPUDS",I1,"r.txt")') trim(fluxobs_directory), ista
     ! write(flnm,'("/avn2/kmanning/STATS/IHOP_DATA/Sites/LE_corrected-fromSEB/&
     ! &IHOPUDS",I1,"r.txt")') ista
  else if (ista == 10) then
     write(flnm, fmt='(A,"/IHOPUDCUS",I2,"r.txt")') trim(fluxobs_directory), ista
     ! write(flnm,'("/avn2/kmanning/STATS/IHOP_DATA/Sites/LE_corrected-fromSEB/&
     ! &IHOPUDCUS10r.txt")')
  endif

  open(iunit, file=trim(flnm), status='old', form='formatted', action='read', iostat=ierr)
  if (ierr /= 0) then
     write(*,'("***** ERROR EXIT *****",/)')
     write(*,'(3x, "Problem opening flux observations file:", /, 6x, A,/)') "'"//trim(flnm)//"'"
     write(*,'("***** ERROR EXIT *****")')
     stop
  endif

  select case (ista)
  case (1)
     lat = 36.0 + 28.370/60.
     lon = -(100.0 + 37.075/60.)
  case (2)
     lat = 36.0 + 37.327/60.
     lon = -(100.0 + 37.619/60.)
  case (3)
     lat = 36.0 + 51.662/60.
     lon = -(100.0 + 35.670/60.)
  case (4)
     lat = 37.0 + 21.474/60.
     lon = -(98.0 + 14.679/60.)
  case (5)
     lat = 37.0 + 22.684/60.
     lon = -(98.0 + 9.816/60.)
  case (6)
     lat = 37.0 + 21.269/60.
     lon = -(97.0 + 39.200/60.)
  case (7)
     lat = 37.0 + 18.792/60.
     lon = -(96.0 + 56.323/60.)
  case (8)
     lat = 37.0 + 24.418/60.
     lon = -(96.0 + 45.937/60.)
  case(9)
     lat = 37.0 + 24.618/60.
     lon = -(96.0 + 34.028/60.)
  case (10)
     ! From Joe Alfieri
     lat =  36.8764333333
     lon = -100.60890111
  end select


  ! Skip 4 header lines
  read(iunit,'(///)')
  READLOOP : do 

!
! Fore some reason, I'm having a hard time reading this file with
! the Intel compiler.  Mabye it's fixed in a later version.  For now,
! stick with pgi.
!
! Anyway, the read into character string Hrd_LEc is done because some bad data
! values in the file are encoded as the string "NaN".  Arrgh!
!
        

     if (ista == 10) then
        read(iunit,*, iostat=ierr) idum, idum, idum, idum, idum, rdyyyy, &
             rdmm, rddd, rdhhmmss, idum, idum, rd_rsw, rd_rlw, rt_T, rd_P, rd_MR, &
             rd_SPD, rd_U, rd_V, rd_rainacc, rd_rainr, rd_Rnet, rd_Rpar, &
             rd_Tsfc, rd_LE, Hrd_LEc, rd_H, rd_Gsfc 
     else
        read(iunit,*, iostat=ierr) idum, idum, idum, idum, idum, rdyyyy, &
              rdmm, rddd, rdhhmmss, idum, rd_rsw, rd_rlw, rt_T, rd_P, rd_MR, &
              rd_SPD, rd_U, rd_V, rd_rainacc, rd_rainr, rd_Rnet, rd_Rpar, &
              rd_Tsfc, rd_LE, Hrd_LEc, rd_H, rd_Gsfc
     endif
     if (ierr == -1) then
        print*, 'Hit end of file.'
        exit
     endif
     if (ierr /= 0) then
        print*, 'Hit read error.', ierr
        stop
     endif
     write(rddate, '(I4.4,2("-",I2.2),"_",I2.2,":",I2.2)') rdyyyy, rdmm, rddd, &
          rdhhmmss/10000, mod(rdhhmmss/100, 100)
     print '(I2, 2x, A, 3F20.10)', ista, rddate, rd_H, rd_le, rd_lec
     if (rddate(1:16) >= startdate(1:16)) then
        call geth_idts(rddate(1:16), startdate(1:16), itime)
        print*, 'rddate, startdate = ', rddate, '  ', startdate
        itime = (itime / 15) + 1
        print*, 'itime = ', itime
        if (itime > maxtim) stop "Increase maxtim."
!     if (rd_rlw < 9998) lw(itime) = rd_rlw
!     if (rd_rsw < 9998) sw(itime) = rd_rsw

        if (rd_H < 9998) HFX(itime) = rd_H
        if (rd_LE < 9998) QFX_OLD(itime) = rd_LE
        if (Hrd_LEc(1:3) /= "NaN") then
           read(Hrd_LEc, '(F7.0)', iostat=ierr) rd_lec
           if (ierr == 0) then
              if (rd_LEc < 9998) QFX_CORRECTED(itime) = rd_LEc
           else
              stop "Problem converting rd_LEc"
           endif
        endif

        hdate(itime) = rddate
!KWM     print*, rdyyyy, rdmm, rddd, rdhhmmss, hdate(icount)
     endif
  enddo READLOOP
  close(iunit)
  print*, 'finish routine read_ihop.'

end subroutine read_ihop

subroutine read_namelist(fluxobs_directory, ldasout_directory)
  implicit none
  character(len=256), intent(out) :: fluxobs_directory
  character(len=256), intent(out) :: ldasout_directory
  namelist/fluxstats/ fluxobs_directory, ldasout_directory

  integer :: ierr

  open(11, file="namelist.fluxstats", status='old', form='formatted', action='read', iostat=ierr)
  if (ierr /= 0) then
     write(*,'("***** ERROR EXIT *****",/)')
     write(*,'(3x, "Problem opening namelist file:", /, 6x, A,/)') "'"//"namelist.fluxstats"//"'"
     write(*,'("***** ERROR EXIT *****")')
     stop
  endif

  read(11, nml=fluxstats, iostat=ierr) 
  if (ierr /= 0) then
     write(*,'("***** ERROR EXIT *****",/)')
     write(*,'(3x, "Problem reading namelist file:", /, 6x, A,/)') "'"//"namelist.fluxstats"//"'"
     write(*,'("***** ERROR EXIT *****")')
     stop
  endif

  close(11)

  print*, 'fluxobs_directory = "'//trim(fluxobs_directory)//'"'
  
end subroutine read_namelist
